package GCG.coloredVertex;

import abiturklassen.graph.Vertex;

/**
 * Ein Knoten im Sinne der NRW-Graph-Klasse kann mit einer beliebigen Farbe,
 * charakterisiert durch eine positive Zahl, gefaerbt werden.
 * 
 * @version 5.0 vom 20.12.2015
 * @author Klaus Bovermann
 */
public class ColoredVertex extends Vertex {

	public static final int KEINE_FARBE = -1;
	private int colorNr;

	/**
	 * ein neuer Knoten wird erzeugt. Er hat keine Farbe.
	 * 
	 * @param pId
	 *            Die eindeutige ID des Knotens
	 */
	public ColoredVertex(String pId) {
		super(pId);
		this.colorNr = KEINE_FARBE;
	}

	/**
	 * liefert genau dann true, wenn der Knoten eine von -1 verschiedene Farbe
	 * hat.
	 * 
	 * @return hat der Knoten eine korrekte Farbe?
	 */
	public boolean isColored() {
		return this.getColor() != KEINE_FARBE;
	}

	/**
	 * liefert die Farbnummer, mit dem der Knoten gefaerbt wurde. Hat der Knoten
	 * noch keine Farbe, wird -1 zurueckgeliefert.
	 * 
	 * @return die Farbe des Knoten
	 */
	public int getColor() {
		return this.colorNr;
	}

	/**
	 * Der Knoten wird mit der angegebenen Farbe gefaerbt. Es kann hiermit
	 * jedoch nicht der Wert KEINE_FARBE gesetzt werden. Es wird ansonsten keine
	 * Ueberpruefung des Parameters vorgenommen.
	 * 
	 * @param pColorNr
	 *            Die Nummer der Farbe
	 */
	public void setColor(int pColorNr) {
		if (pColorNr != KEINE_FARBE) {
			this.colorNr = pColorNr;
		}
	}

	/**
	 * Der Knoten hat anschliessend keine Farbe.
	 */
	public void entfaerbe() {
		this.colorNr = KEINE_FARBE;
	}
}
