package GCG.server;

import java.io.IOException;

import GCG.coloredVertex.ColoredVertexGraphConstructor;
import GCG.protokoll.PROTOKOLL;
import abiturklassen.graph.Graph;
import abiturklassen.netz.Server;

/**
 * Die drei zu implementierenden Methoden wurden synchronisiert
 * 
 * @version 3.2 vom 16.8.2016
 * @author Klaus Bovermann
 */
public class GCG_Server extends Server {

	private ColoredVertexGraphConstructor gb;
	private Graph meinGraph;
	private GCG_Spielfeld dasSpielfeld;
	private ZweiSpieler dieBeidenSpieler;
	private EinSpieler aktiverSpieler;
	// private EinSpieler inaktiverSpieler;
	private int anzahlFarben;

	private GCG_ServerFrame meineAusgabe;

	/**
	 * @param pPortNr
	 *            Der Server wird auf diesem Port angesprochen
	 * @param pFilename
	 *            Der Name der Datei, in der die Kanteninformationen enthalten
	 *            sind
	 * @param pAnzahlFarben
	 *            Die Anzahl der Farben, die in diesem Spiel erlaubt sind.
	 */
	public GCG_Server(int pPortNr, String pFilename, int pAnzahlFarben) {
		super(pPortNr);

		gb = new ColoredVertexGraphConstructor();

		try {
			gb.fileToGraph(pFilename);
		} catch (IOException e) {
			e.printStackTrace();
		}

		this.meinGraph = gb.getGraph();

		this.dasSpielfeld = new GCG_Spielfeld(this.meinGraph, pAnzahlFarben);

		this.dieBeidenSpieler = new ZweiSpieler();

		this.anzahlFarben = pAnzahlFarben;

		this.aktiverSpieler = null;
		// this.inaktiverSpieler = null;

		this.meineAusgabe = new GCG_ServerFrame();

		this.meineAusgabe.ausgeben("Server gestartet!");
		this.meineAusgabe.setVisible(true);
	}

	private void okSenden(String pClientIP, int pClientPort, String okString) {
		this.senden(pClientIP, pClientPort, okNachricht(okString));
	}

	private void errorSenden(String pClientIP, int pClientPort, String errorString) {
		this.senden(pClientIP, pClientPort, fehlerNachricht(errorString));
	}

	@Override
	public /*synchronized*/ void processNewConnection(String pClientIP, int pClientPort) {
		if (this.dieBeidenSpieler.getErster() == null) {
			this.dieBeidenSpieler.setErsterSpieler(pClientIP, pClientPort);
			this.dieBeidenSpieler.getErster().setName("1. Spieler");
			this.okSenden(pClientIP, pClientPort, "1. Anmeldung erfolgreich!");
			this.senden(pClientIP, pClientPort, nameNachricht(this.dieBeidenSpieler.getErster().getName()));
		} else if (this.dieBeidenSpieler.getZweiter() == null) {
			this.dieBeidenSpieler.setZweiterSpieler(pClientIP, pClientPort);
			this.okSenden(pClientIP, pClientPort, "2. Anmeldung erfolgreich!");
			this.dieBeidenSpieler.getZweiter().setName("2. Spieler");
			this.senden(pClientIP, pClientPort, nameNachricht(this.dieBeidenSpieler.getZweiter().getName()));
			// Jetzt sind beide Spieler angemeldet!
			// Also kann es jetzt losgehen!
			this.dieBeidenSpieler.start();
			this.aktiverSpieler = this.dieBeidenSpieler.getAktiverSpieler();
			// this.inaktiverSpieler =
			// this.dieBeidenSpieler.getInaktiverSpieler();
			sendeStartnachrichten();
		} else { // beide sind schon registriert, also Spiel hat begonnen!
			this.closeConnection(pClientIP, pClientPort);
		}
	}

	@Override
	public /*synchronized*/ void processMessage(String pClientIP, int pClientPort, String pMessage) {
		String spielerName = this.dieBeidenSpieler.getName(pClientIP, pClientPort);
		this.meineAusgabe.empfangAusgeben(spielerName, pMessage);
		if (this.dieBeidenSpieler.istSpielGestartet()) {

			String[] nachrichtenTeile = pMessage.split(PROTOKOLL.TRENNER);
			String anfang = nachrichtenTeile[0];

			switch (anfang) {
			case PROTOKOLL.CS_KNOTEN:
				knotenSchicken(pClientIP, pClientPort);
				break;
			case PROTOKOLL.CS_GRAPH:
				graphSchicken(pClientIP, pClientPort);
				break;
			case PROTOKOLL.CS_FAERBE:
				if (!pClientIP.equals(this.aktiverSpieler.getIp()) || pClientPort != this.aktiverSpieler.getPort()) {
					this.errorSenden(pClientIP, pClientPort, "Sie sind nicht am Zug!");
				} else if (nachrichtenTeile.length == 3) {
					farbVersuch(nachrichtenTeile[1], nachrichtenTeile[2], pClientIP, pClientPort);
				} else {
					this.errorSenden(pClientIP, pClientPort, "Falsche Syntax!");
				}
				break;
			case PROTOKOLL.CS_FARBZAHL:
				farbenSchicken(pClientIP, pClientPort);
				break;
			default:
				this.errorSenden(pClientIP, pClientPort, "Unbekannte Nachricht!");
			}
		} else {
			this.errorSenden(pClientIP, pClientPort, "Spiel noch nicht gestartet!");
		}
	}

	private void farbVersuch(String knotenName, String farbNrString, String pClientIP, int pClientPort) {
		int farbNr = Integer.parseInt(farbNrString);
		// ueberpruefe die Korrektheit aller Angaben
		int fehlercode = this.dasSpielfeld.istErlaubteFaerbung(knotenName, farbNr);
		if (fehlercode == 0) // vorl. !!!!!!!!!!
		{
			this.senden(pClientIP, pClientPort, inaktivNachricht());
			this.allenSenden(farbNachricht(knotenName, farbNr));

			if (this.dasSpielfeld.alleKnotenGefaerbt()) {
				this.allenSenden(remisNachricht("Alle Knoten sind gefaerbt!"));
			} else if (this.dasSpielfeld.keinWeitererKnotenFaerbbar()) {
				this.allenSenden(spielEndeMitGewinnerNachricht(this.aktiverSpieler.getName()));
			} else {
				this.dieBeidenSpieler.switchAktiverSpieler();
				this.aktiverSpieler = this.dieBeidenSpieler.getAktiverSpieler();
				// this.inaktiverSpieler =
				// this.dieBeidenSpieler.getInaktiverSpieler();
				this.senden(this.aktiverSpieler.getIp(), this.aktiverSpieler.getPort(), aktivNachricht());
			}

		} else if (fehlercode == 1) {
			this.errorSenden(pClientIP, pClientPort, "Knoten nicht bekannt!");
		} else if (fehlercode == 2) {
			this.errorSenden(pClientIP, pClientPort, "Knoten bereits gefaerbt!");
		} else if (fehlercode == 3) {
			this.errorSenden(pClientIP, pClientPort, "Nachbar hat gleiche Farbe!");
		} else if (fehlercode == 4) {
			this.errorSenden(pClientIP, pClientPort, "Unzulaessige Farbe!");
		} else { // sollte nie passieren!
			this.errorSenden(pClientIP, pClientPort, "Unbekannter Fehler!");
		}

	}

	private void graphSchicken(String pIP, int pPort) {
		this.senden(pIP, pPort, graphNachricht(this.gb.getGraphString()));
	}

	private void knotenSchicken(String pIP, int pPort) {
		if (this.dieBeidenSpieler.istSpielGestartet()) {
			this.senden(pIP, pPort, knotenNachricht(this.dasSpielfeld.verticesToString()));
		} else {
			this.errorSenden(pIP, pPort, "Spiel noch nicht gestartet!");
		}
	}

	private void farbenSchicken(String pIP, int pPort) {
		this.senden(pIP, pPort, farbanzahlNachricht(this.anzahlFarben));
	}

	@Override
	public synchronized void processClosingConnection(String pClientIP, int pClientPort) {
		this.senden(pClientIP, pClientPort, beendeNachricht("Verbindung wird beendet"));
	}

	private String nachricht(String tag) {
		return tag;
	}

	private String nachricht(String tag, String inhalt) {
		return tag + PROTOKOLL.TRENNER + inhalt;
	}

	private String nachricht(String tag, String inhalt_1, String inhalt_2) {
		return tag + PROTOKOLL.TRENNER + inhalt_1 + PROTOKOLL.TRENNER + inhalt_2;
	}

	private String farbNachricht(String knoten, int farbnr) {
		return nachricht(PROTOKOLL.SC_GEFAERBT, knoten, "" + farbnr);
	}

	private String knotenNachricht(String dieKnoten) {
		return nachricht(PROTOKOLL.SC_KNOTEN, dieKnoten);
	}

	private String nameNachricht(String derName) {
		return nachricht(PROTOKOLL.SC_NAME, derName);
	}

	private String graphNachricht(String derGraph) {
		return nachricht(PROTOKOLL.SC_GRAPH, derGraph);
	}

	private String farbanzahlNachricht(int anzahl) {
		return nachricht(PROTOKOLL.SC_FARBZAHL, "" + anzahl);
	}

	private String fehlerNachricht(String fehler) {
		return nachricht(PROTOKOLL.SC_ERROR, fehler);
	}

	private String remisNachricht(String zusatz) {
		return nachricht(PROTOKOLL.SC_UNENTSCHIEDEN, zusatz);
	}

	private String spielEndeMitGewinnerNachricht(String siegername) {
		return nachricht(PROTOKOLL.SC_GEWINN_ENDE, siegername);
	}

	private String beendeNachricht(String meldung) {
		return nachricht(PROTOKOLL.SC_VERBINDUNG_BEENDET, meldung);
	}

	private String okNachricht(String meldung) {
		return nachricht(PROTOKOLL.SC_OK, meldung);
	}

	private String startNachricht() {
		return nachricht(PROTOKOLL.SC_START);
	}

	private String aktivNachricht() {
		return nachricht(PROTOKOLL.SC_AKTIV);
	}

	private String inaktivNachricht() {
		return nachricht(PROTOKOLL.SC_INAKTIV);
	}

	private void allenSenden(String nachricht) {
		this.sendToAll(nachricht);
		this.meineAusgabe.globalsendungAusgeben(nachricht);
	}

	private void senden(String pIP, int pPort, String nachricht) {
		this.send(pIP, pPort, nachricht);
		String spielerName = this.dieBeidenSpieler.getName(pIP, pPort);
		this.meineAusgabe.einzelsendungAusgeben(spielerName, nachricht);
	}

	private void sendeStartnachrichten() {
		this.allenSenden(startNachricht());
		this.allenSenden(graphNachricht(this.gb.getGraphString()));
		this.allenSenden(farbanzahlNachricht(this.anzahlFarben));
		this.senden(this.aktiverSpieler.getIp(), this.aktiverSpieler.getPort(), aktivNachricht());
	}
}
