package GCG.server;

import GCG.coloredVertex.ColoredVertex;
import abiturklassen.graph.Graph;
import abiturklassen.graph.Vertex;
import abiturklassen.linear.List;

/**
 * 
 * @version 3.1 vom 4.11.2015
 * @author Klaus Bovermann
 *
 */
public class GCG_Spielfeld {

	/**
	 * Dieser Graph benutzt Knoten vom Typ ColoredVertex. Man muss also beim
	 * Zugriff auf einen solchen Knoten das zurueckgelieferte Objekt geeignet
	 * casten!
	 */
	private Graph derGraph;
	private int anzahlFarben;

	/**
	 * @param pDerGraph
	 * @param pAnzahlFarben
	 */
	public GCG_Spielfeld(Graph pDerGraph, int pAnzahlFarben) {
		this.derGraph = pDerGraph;
		this.anzahlFarben = pAnzahlFarben;
	}

	/**
	 * @return liefert genau dann true, wenn es keinen Knoten mehr gibt, der
	 *         korrekt faerbbar ist
	 */
	public boolean keinWeitererKnotenFaerbbar() {
		List<Vertex> alleKnoten = this.derGraph.getVertices();
		boolean gefunden = false;
		alleKnoten.toFirst();
		while (!gefunden && alleKnoten.hasAccess()) {
			ColoredVertex dieser = (ColoredVertex) alleKnoten.getContent();
			if (!dieser.isColored() && hatFaerbbarenNachbarn(dieser)) {
				gefunden = true;
			} else {
				alleKnoten.next();
			}
		}
		return !gefunden;
	}

	/**
	 * @param pKnoten
	 * @return liefert genau dann true, wenn ein Nachbarknoten korrekt zu
	 *         faerben ist
	 */
	private boolean hatFaerbbarenNachbarn(ColoredVertex pKnoten) {
		List<Vertex> nachbarn = this.derGraph.getNeighbours(pKnoten);
		Boolean[] farben = new Boolean[this.anzahlFarben];
		// man hat die Farben 0, 1, 2, 3, ... sowie als Unfarbe die Farbe-1
		for (int i = 0; i < this.anzahlFarben; i++) {
			farben[i] = false;
		}
		nachbarn.toFirst();
		while (nachbarn.hasAccess()) {
			ColoredVertex cv = (ColoredVertex) nachbarn.getContent();
			if (cv.isColored()) {
				int farbe = cv.getColor();
				farben[farbe] = true;
			}
			nachbarn.next();
		}
		boolean gefunden = false;
		for (int i = 0; i < this.anzahlFarben && !gefunden; i++) {
			if (!farben[i]) {
				gefunden = true;
			}
		}
		return gefunden;
	}

	/**
	 * Ein String der Form KNOTEN/FARBE;KNOTEN/FARBE; ... ; KNOTEN/FARBE
	 * 
	 * @return die textuelle Beschreibung aller gefaerbten Knoten
	 */
	public String verticesToString() {
		String result = "";
		List<Vertex> alleKnoten = this.derGraph.getVertices();
		alleKnoten.toFirst();
		while (alleKnoten.hasAccess()) {
			ColoredVertex dieser = (ColoredVertex) alleKnoten.getContent();
			String name = dieser.getID();
			int farbe = dieser.getColor();
			if (!result.equals("")) {
				result += ";";
			}
			result += name + "/" + farbe;
			alleKnoten.next();
		}
		return result;
	}

	/**
	 * @return liefert genau dann true, wenn alle Knoten im Graph gefaerbt sind
	 */
	public boolean alleKnotenGefaerbt() {
		boolean alleSindFarbig = true;
		List<Vertex> alleKnoten = this.derGraph.getVertices();
		alleKnoten.toFirst();
		while (alleSindFarbig && alleKnoten.hasAccess()) {
			ColoredVertex dieser = (ColoredVertex) alleKnoten.getContent();
			if (!dieser.isColored()) {
				alleSindFarbig = false;
			} else {
				alleKnoten.next();
			}
		}
		return alleSindFarbig;
	}

	/**
	 * 
	 * @param knotenName
	 *            Dieser Knoten soll gefaerbt werden
	 * @param farbNr
	 *            mit dieser Farbe soll der Knoten gefaerbt werden
	 * @return Fehlercodierung:
	 * 
	 *         0 = kein Fehler;
	 * 
	 *         1 = Knoten gibt es nicht;
	 * 
	 *         2 = Knoten bereits gefaerbt;
	 * 
	 *         3 = Knoten hat identische Farbe mit der eines Nachbarn
	 * 
	 *         4 = Farbnummer unzulaessig
	 */
	public int istErlaubteFaerbung(String knotenName, int farbNr) {

		if (farbNr < 0 || farbNr >= this.anzahlFarben) {
			return 4;
		}
		// ist er ein bekannter KnotenName?
		List<Vertex> alleKnoten = this.derGraph.getVertices();
		ColoredVertex dieser = null;
		alleKnoten.toFirst();
		boolean gefunden = false;
		while (!gefunden && alleKnoten.hasAccess()) {
			dieser = (ColoredVertex) alleKnoten.getContent();
			if (dieser.getID().equals(knotenName)) {
				gefunden = true;

			} else {
				alleKnoten.next();
			}
		}

		if (!gefunden) {
			return 1;
		} else {
			if (dieser.isColored()) { // Knoten bereits gefaerbt!
				return 2;
			} else {
				// sind alle Nachbarn anders gefaerbt??
				List<Vertex> alleNachbarn = this.derGraph.getNeighbours(dieser);
				alleNachbarn.toFirst();
				boolean ok = true;
				while (ok && alleNachbarn.hasAccess()) {
					ColoredVertex nachbar = (ColoredVertex) alleNachbarn.getContent();
					int farbNrVonNachbar = nachbar.getColor();
					if (farbNrVonNachbar == farbNr) {
						ok = false;
					} else {
						alleNachbarn.next();
					}
				}
				if (ok) {
					dieser.setColor(farbNr);
					return 0;
				} else {
					return 3;
				}
			}
		}
	}
}
